#
# Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_agent.parsers.sonic.redis_db.nvos_client import NvosDBRender
from netq_agent.parsers.sonic.redis_db.sonic_db import SonicDB, SonicTable
from netq_lib.orm.redisdb.models import IbFan, IbPsu, IbAsic, IbPlatform, IbCpu


def _attr_state_db_fan_key_mapping():
    return {
        'fan_state': 'status',
        'fan_speed': 'speed',
    }


def _attr_state_db_asic_key_mapping():
    return {
        'asic_temp': 'temperature'
    }


def _attr_state_db_psu_key_mapping():
    return {
        'psu_state': 'status',
        'input_current': 'input_current',
        'input_voltage': 'input_voltage',
        'output_current': 'current',
        'output_voltage': 'voltage',
        'output_power': 'power',
    }


class PlatformHandler(NvosDBRender):
    def __init__(self):
        super(PlatformHandler, self).__init__()

    def db_sources(self):
        return {
            (SonicDB.CONFIG_DB, SonicTable.CONFIG_DB_DEVICE_METADATA): self.default_callback,
            (SonicDB.STATE_DB, SonicTable.STATE_DB_AMBIENT_TEMPERATURE): self.default_callback,
            (SonicDB.STATE_DB, SonicTable.STATE_DB_CPU_INFO_UTIL): self.default_callback,
            (SonicDB.STATE_DB, SonicTable.STATE_DB_DISK_INFO_USAGE): self.default_callback,
            (SonicDB.STATE_DB, SonicTable.STATE_DB_RAM_INFO_USAGE): self.default_callback,
            (SonicDB.STATE_DB, SonicTable.STATE_DB_FAN_INFO): self.default_callback,
            (SonicDB.STATE_DB, SonicTable.STATE_DB_PSU_INFO): self.default_callback,
            (SonicDB.STATE_DB, SonicTable.STATE_DB_TEMPERATURE_INFO): self.default_callback,
        }

    def update_obj(self, obj, data, attr_map):
        if data:
            for attr_name, db_key in attr_map.items():
                try:
                    setattr(obj, attr_name, data.get(db_key, ""))
                except (ValueError, KeyError) as e:
                    self._logger.debug(
                        'Failed to set attr {} for object. error: {}'.format(attr_name, str(e)))

    def process_fullstate(self, data):
        self.process_fans(data)
        self.process_psus(data)
        self.process_asics(data)
        self.process_platform_general(data)
        self.process_cpu(data)

    def process_fans(self, data):
        fans = self.data_get(data, SonicDB.STATE_DB, SonicTable.STATE_DB_FAN_INFO)
        for fan in fans:
            ib_fan_obj = IbFan()
            ib_fan_obj.name = fan
            fan_data = fans.get(fan)
            self.update_obj(ib_fan_obj, fan_data, _attr_state_db_fan_key_mapping())
            ib_fan_obj.save()

    def process_psus(self, data):
        psus = self.data_get(data, SonicDB.STATE_DB, SonicTable.STATE_DB_PSU_INFO)
        for psu in psus:
            ib_psu_obj = IbPsu()
            ib_psu_obj.name = psu
            psu_data = psus.get(psu)
            self.update_obj(ib_psu_obj, psu_data, _attr_state_db_psu_key_mapping())
            ib_psu_obj.save()

    def process_asics(self, data):
        temperature_info = self.data_get(data, SonicDB.STATE_DB, SonicTable.STATE_DB_TEMPERATURE_INFO)
        asics = {key: value for key, value in temperature_info.items() if "asic" in key.lower()}
        for asic in asics:
            ib_asic_obj = IbAsic()
            ib_asic_obj.name = asic
            asic_data = asics.get(asic)
            self.update_obj(ib_asic_obj, asic_data, _attr_state_db_asic_key_mapping())
            ib_asic_obj.save()

    def process_platform_general(self, data):
        device_metadata = self.data_get(data, SonicDB.CONFIG_DB, SonicTable.CONFIG_DB_DEVICE_METADATA)
        ambient_port_side = self.data_get(data, SonicDB.STATE_DB, SonicTable.STATE_DB_AMBIENT_TEMPERATURE)
        disk_info = self.data_get(data, SonicDB.STATE_DB, SonicTable.STATE_DB_DISK_INFO_USAGE)
        ram_info = self.data_get(data, SonicDB.STATE_DB, SonicTable.STATE_DB_RAM_INFO_USAGE)
        ib_platform_obj = IbPlatform()
        ib_platform_obj.platform_name = device_metadata.get("platform", "")
        ib_platform_obj.ambient_temperature = ambient_port_side.get("temperature", "")
        ib_platform_obj.disk_total_size = disk_info.get("disk_total_size", "")
        ib_platform_obj.disk_used = disk_info.get("disk_usage", "")
        ib_platform_obj.memory_total_size = ram_info.get("memory_total_size", "")
        ib_platform_obj.memory_used = ram_info.get("memory_usage", "")
        ib_platform_obj.save()

    def process_cpu(self, data):
        cpu_info = self.data_get(data, SonicDB.STATE_DB, SonicTable.STATE_DB_CPU_INFO_UTIL)
        ib_cpu_obj = IbCpu()
        if not cpu_info:
            ib_cpu_obj.state = "failed"
        else:
            ib_cpu_obj.state = "ok"
            ib_cpu_obj.utilization = cpu_info.get("utilization", "")
        ib_cpu_obj.save()


def process_platform(*args, **kwargs):
    logger = kwargs.get('logger')
    render.process(logger)


render = PlatformHandler()
